// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.content.res.Resources;
import android.graphics.drawable.Drawable;
import android.view.Gravity;
import android.view.ViewGroup.LayoutParams;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.appharbr.sdk.engine.AdSdk;
import com.appharbr.sdk.engine.AppHarbr;
import com.appharbr.sdk.engine.listeners.AHListener;
import com.applovin.mediation.ads.MaxAdView;
import com.applovin.sdk.AppLovinSdk;
import com.applovin.sdk.AppLovinSdkUtils;

import io.flutter.plugin.platform.PlatformView;
import io.flutter.util.Preconditions;

/**
 * A wrapper for {@link MaxAdView}.
 */
class FlutterAppLovinBannerAd extends FlutterAd implements FlutterAdLoadedListener {

    @NonNull
    protected final AdInstanceManager manager;
    @NonNull
    private final String adUnitId;
    @NonNull
    private final Boolean isMREC;
    @NonNull
    private final Boolean isAppharbarSuccess;
    @NonNull
    private final FlutterAdSize size;
    @NonNull
    private final AppLovinSdk _appLovinSdk;
    @NonNull
    private final BannerAdCreator bannerAdCreator;
    @Nullable
    protected MaxAdView adView;

    /**
     * Constructs a `FlutterAdManagerBannerAd`.
     *
     * <p>Call `load()` to instantiate the `AdView` and load the `AdRequest`. `getView()` will return
     * null only until `load` is called.
     */
    public FlutterAppLovinBannerAd(
            int adId,
            @NonNull AdInstanceManager manager,
            @NonNull String adUnitId,
            @NonNull Boolean isMREC,
            @NonNull FlutterAdSize size,
            @NonNull AppLovinSdk _appLovinSdk,
            @NonNull Boolean isAppharbarSuccess,
            @NonNull BannerAdCreator bannerAdCreator) {
        super(adId);
        Preconditions.checkNotNull(manager);
        Preconditions.checkNotNull(adUnitId);
        Preconditions.checkNotNull(size);
        Preconditions.checkNotNull(_appLovinSdk);
        this.manager = manager;
        this.adUnitId = adUnitId;
        this.isMREC = isMREC;
        this._appLovinSdk = _appLovinSdk;
        this.isAppharbarSuccess = isAppharbarSuccess;
        this.size = size;
        this.bannerAdCreator = bannerAdCreator;
    }

    @Override
    void load() {
        adView = this.isMREC ? bannerAdCreator.createMRECAppLovinAdView(adUnitId, _appLovinSdk) : bannerAdCreator.createAppLovinAdView(adUnitId, _appLovinSdk);

        if (adView == null) {
            return;
        }

        int width  = this.isMREC ? AppLovinSdkUtils.dpToPx(manager.getActivity(), size.getAdSize().getWidth() ) : LayoutParams.MATCH_PARENT;
        int height = this.isMREC ? AppLovinSdkUtils.dpToPx(manager.getActivity(), size.getAdSize().getHeight() ) : (int) (size.getAdSize().getHeight() * Resources.getSystem().getDisplayMetrics().density);
        adView.setGravity(Gravity.CENTER);
        adView.setLayoutParams(new LayoutParams(width, height));
        adView.setListener(new FlutterBannerAdListener(adId, manager, this));

        if (isAppharbarSuccess) {
            AHListener ahListener = (o, s, adFormat, adBlockReasons) -> {};
            AppHarbr.addBannerView(AdSdk.MAX, adView, null, ahListener);
        }

        adView.loadAd();
//        adView.startAutoRefresh();
    }

    @Override
    public void onAdLoaded() {
        if (adView != null) {
            manager.onAdLoaded(adId, null);
        }
    }

    @Nullable
    @Override
    PlatformView getPlatformView() {
        if (adView == null) {
            return null;
        }
        return new FlutterPlatformView(adView);
    }

    @Override
    void dispose() {
        if (adView != null) {
            adView.destroy();
            adView = null;
        }
    }

    @Nullable
    FlutterAdSize getAdSize() {
        if (adView == null || size.getAdSize() == null) {
            return null;
        }
        return new FlutterAdSize(size.getAdSize());
    }

    @Override
    public void onAdLoaded(@NonNull Drawable bitmap) {
    }

    @Override
    public void onAdLoadFailed() {

    }
}
